Re: [PATCH v2] target/ppc: Fix for optimized vsl/vsr instructions

2019-10-05 Thread David Gibson
On Fri, Oct 04, 2019 at 01:52:41PM -0500, Paul Clarke wrote:
> On 10/4/19 8:43 AM, Stefan Brankovic wrote:
> > In previous implementation, invocation of TCG shift function could request
> > shift of TCG variable by 64 bits when variable 'sh' is 0, which is not
> > supported in TCG (values can be shifted by 0 to 63 bits). This patch fixes
> > this by using two separate invocation of TCG shift functions, with maximum
> > shift amount of 32.
> > 
> > Name of variable 'shifted' is changed to 'carry' so variable naming
> > is similar to old helper implementation.
> > 
> > Variables 'avrA' and 'avrB' are replaced with variable 'avr'.
> > 
> > Fixes: 4e6d0920e7547e6af4bbac5ffe9adfe6ea621822
> > Reported-by: Paul Clark 
> 
> Preferred: "Paul A. Clarke" (for historical consistency)
> 
> > Reported-by: Mark Cave-Ayland 
> > Suggested-by: Aleksandar Markovic 
> > Signed-off-by: Stefan Brankovic 
> 
> Applying this patch on top of dce5a787c05fe1a3e54d92871cdeba2af6798e0d 
> eliminated the failures that I reported in 
> https://bugs.launchpad.net/qemu/+bug/1841990 associated with vsl/vsr.
> 
> Tested-by: Paul A. Clarke  

I've applied this to ppc-for-4.2, adjusting the Reported-by line as
suggested above.

-- 
David Gibson| I'll have my music baroque, and my code
david AT gibson.dropbear.id.au  | minimalist, thank you.  NOT _the_ _other_
| _way_ _around_!
http://www.ozlabs.org/~dgibson


signature.asc
Description: PGP signature


Re: [PATCH v2] target/ppc: Fix for optimized vsl/vsr instructions

2019-10-04 Thread Paul Clarke
On 10/4/19 8:43 AM, Stefan Brankovic wrote:
> In previous implementation, invocation of TCG shift function could request
> shift of TCG variable by 64 bits when variable 'sh' is 0, which is not
> supported in TCG (values can be shifted by 0 to 63 bits). This patch fixes
> this by using two separate invocation of TCG shift functions, with maximum
> shift amount of 32.
> 
> Name of variable 'shifted' is changed to 'carry' so variable naming
> is similar to old helper implementation.
> 
> Variables 'avrA' and 'avrB' are replaced with variable 'avr'.
> 
> Fixes: 4e6d0920e7547e6af4bbac5ffe9adfe6ea621822
> Reported-by: Paul Clark 

Preferred: "Paul A. Clarke" (for historical consistency)

> Reported-by: Mark Cave-Ayland 
> Suggested-by: Aleksandar Markovic 
> Signed-off-by: Stefan Brankovic 

Applying this patch on top of dce5a787c05fe1a3e54d92871cdeba2af6798e0d 
eliminated the failures that I reported in 
https://bugs.launchpad.net/qemu/+bug/1841990 associated with vsl/vsr.

Tested-by: Paul A. Clarke  

PC



Re: [PATCH v2] target/ppc: Fix for optimized vsl/vsr instructions

2019-10-04 Thread Aleksandar Markovic
> Reported-by: Paul Clark 

Stefan,

Paul's full name is Paul A. Clarke.

Thanks for the fix!

Aleksandar



[PATCH v2] target/ppc: Fix for optimized vsl/vsr instructions

2019-10-04 Thread Stefan Brankovic
This patch fixes bug in optimized vsl/vsr instructions reported by Mark
Cave-Ayland and Paul Clarke. Sorry for not responding earlier, I was absent
last couple of days. I also integrated some suggestions made by Aleksandar
Markovic. New soultion is tested and still has noticable performance
improvement compared to old helper implementation.

V1 of this patch was not sent to qemu-devel and I am now sending V2 to
appropriate email adresses.

Stefan Brankovic (1):
  target/ppc: Fix for optimized vsl/vsr instructions

 target/ppc/translate/vmx-impl.inc.c | 84 ++---
 1 file changed, 40 insertions(+), 44 deletions(-)

-- 
2.7.4




[PATCH v2] target/ppc: Fix for optimized vsl/vsr instructions

2019-10-04 Thread Stefan Brankovic
In previous implementation, invocation of TCG shift function could request
shift of TCG variable by 64 bits when variable 'sh' is 0, which is not
supported in TCG (values can be shifted by 0 to 63 bits). This patch fixes
this by using two separate invocation of TCG shift functions, with maximum
shift amount of 32.

Name of variable 'shifted' is changed to 'carry' so variable naming
is similar to old helper implementation.

Variables 'avrA' and 'avrB' are replaced with variable 'avr'.

Fixes: 4e6d0920e7547e6af4bbac5ffe9adfe6ea621822
Reported-by: Paul Clark 
Reported-by: Mark Cave-Ayland 
Suggested-by: Aleksandar Markovic 
Signed-off-by: Stefan Brankovic 
---
 target/ppc/translate/vmx-impl.inc.c | 84 ++---
 1 file changed, 40 insertions(+), 44 deletions(-)

diff --git a/target/ppc/translate/vmx-impl.inc.c 
b/target/ppc/translate/vmx-impl.inc.c
index 2472a52..81d5a7a 100644
--- a/target/ppc/translate/vmx-impl.inc.c
+++ b/target/ppc/translate/vmx-impl.inc.c
@@ -590,40 +590,38 @@ static void trans_vsl(DisasContext *ctx)
 int VT = rD(ctx->opcode);
 int VA = rA(ctx->opcode);
 int VB = rB(ctx->opcode);
-TCGv_i64 avrA = tcg_temp_new_i64();
-TCGv_i64 avrB = tcg_temp_new_i64();
+TCGv_i64 avr = tcg_temp_new_i64();
 TCGv_i64 sh = tcg_temp_new_i64();
-TCGv_i64 shifted = tcg_temp_new_i64();
+TCGv_i64 carry = tcg_temp_new_i64();
 TCGv_i64 tmp = tcg_temp_new_i64();
 
-/* Place bits 125-127 of vB in sh. */
-get_avr64(avrB, VB, false);
-tcg_gen_andi_i64(sh, avrB, 0x07ULL);
+/* Place bits 125-127 of vB in 'sh'. */
+get_avr64(avr, VB, false);
+tcg_gen_andi_i64(sh, avr, 0x07ULL);
 
 /*
  * Save highest sh bits of lower doubleword element of vA in variable
  * shifted and perform shift on lower doubleword.
  */
-get_avr64(avrA, VA, false);
-tcg_gen_subfi_i64(tmp, 64, sh);
-tcg_gen_shr_i64(shifted, avrA, tmp);
-tcg_gen_andi_i64(shifted, shifted, 0x7fULL);
-tcg_gen_shl_i64(avrA, avrA, sh);
-set_avr64(VT, avrA, false);
+get_avr64(avr, VA, false);
+tcg_gen_subfi_i64(tmp, 32, sh);
+tcg_gen_shri_i64(carry, avr, 32);
+tcg_gen_shr_i64(carry, carry, tmp);
+tcg_gen_shl_i64(avr, avr, sh);
+set_avr64(VT, avr, false);
 
 /*
  * Perform shift on higher doubleword element of vA and replace lowest
- * sh bits with shifted.
+ * 'sh' bits with 'carry'.
  */
-get_avr64(avrA, VA, true);
-tcg_gen_shl_i64(avrA, avrA, sh);
-tcg_gen_or_i64(avrA, avrA, shifted);
-set_avr64(VT, avrA, true);
+get_avr64(avr, VA, true);
+tcg_gen_shl_i64(avr, avr, sh);
+tcg_gen_or_i64(avr, avr, carry);
+set_avr64(VT, avr, true);
 
-tcg_temp_free_i64(avrA);
-tcg_temp_free_i64(avrB);
+tcg_temp_free_i64(avr);
 tcg_temp_free_i64(sh);
-tcg_temp_free_i64(shifted);
+tcg_temp_free_i64(carry);
 tcg_temp_free_i64(tmp);
 }
 
@@ -639,39 +637,37 @@ static void trans_vsr(DisasContext *ctx)
 int VT = rD(ctx->opcode);
 int VA = rA(ctx->opcode);
 int VB = rB(ctx->opcode);
-TCGv_i64 avrA = tcg_temp_new_i64();
-TCGv_i64 avrB = tcg_temp_new_i64();
+TCGv_i64 avr = tcg_temp_new_i64();
 TCGv_i64 sh = tcg_temp_new_i64();
-TCGv_i64 shifted = tcg_temp_new_i64();
+TCGv_i64 carry = tcg_temp_new_i64();
 TCGv_i64 tmp = tcg_temp_new_i64();
 
-/* Place bits 125-127 of vB in sh. */
-get_avr64(avrB, VB, false);
-tcg_gen_andi_i64(sh, avrB, 0x07ULL);
+/* Place bits 125-127 of vB in 'sh'. */
+get_avr64(avr, VB, false);
+tcg_gen_andi_i64(sh, avr, 0x07ULL);
 
 /*
  * Save lowest sh bits of higher doubleword element of vA in variable
  * shifted and perform shift on higher doubleword.
  */
-get_avr64(avrA, VA, true);
-tcg_gen_subfi_i64(tmp, 64, sh);
-tcg_gen_shl_i64(shifted, avrA, tmp);
-tcg_gen_andi_i64(shifted, shifted, 0xfe00ULL);
-tcg_gen_shr_i64(avrA, avrA, sh);
-set_avr64(VT, avrA, true);
+get_avr64(avr, VA, true);
+tcg_gen_subfi_i64(tmp, 32, sh);
+tcg_gen_shli_i64(carry, avr, 32);
+tcg_gen_shl_i64(carry, carry, tmp);
+tcg_gen_shr_i64(avr, avr, sh);
+set_avr64(VT, avr, true);
 /*
  * Perform shift on lower doubleword element of vA and replace highest
- * sh bits with shifted.
+ * 'sh' bits with 'carry'.
  */
-get_avr64(avrA, VA, false);
-tcg_gen_shr_i64(avrA, avrA, sh);
-tcg_gen_or_i64(avrA, avrA, shifted);
-set_avr64(VT, avrA, false);
+get_avr64(avr, VA, false);
+tcg_gen_shr_i64(avr, avr, sh);
+tcg_gen_or_i64(avr, avr, carry);
+set_avr64(VT, avr, false);
 
-tcg_temp_free_i64(avrA);
-tcg_temp_free_i64(avrB);
+
@@ -590,40 +590,38 @@ static void trans_vsl(DisasContext *ctx)
 
 /*
- * Save highest sh bits of lower doubleword element of vA in variable
- * shifted and perform shift on lower doubleword.
+ * Save highest 'sh' bits of lower doubleword element of vA in variable
+ * 'carry' and perform shift on lower doubleword.
  */
@@ -639,39 +637,37 @@ static void trans_vsr(DisasContext *ctx)
 
 /*
- * Save lowest sh bits of higher doubleword element of vA in variable
- * shifted and perform shift on higher doubleword.
+ * Save lowest 'sh' bits of higher doubleword element of vA in variable
+ * 'carry' and perform shift on higher doubleword.
  */